@@ -1,7 +1,10 @@
 /* -*- c-file-style: "linux" -*-
+
+   rsync -- fast file replication program
    
    Copyright (C) 1996-2000 by Andrew Tridgell 
    Copyright (C) Paul Mackerras 1996
+   Copyright (C) 2002 by Martin Pool <mbp@samba.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,6 +31,7 @@ extern int am_root;
 extern int preserve_devices;
 extern int preserve_hard_links;
 extern int update_only;
+extern int opt_ignore_existing;
 extern int whole_file;
 extern int block_size;
 extern int csum_length;
@@ -99,25 +103,30 @@ static int adapt_block_size(struct file_struct *file, int bsize)
 /*
   send a sums struct down a fd
   */
-static void send_sums(struct sum_struct *s,int f_out)
+static void send_sums(struct sum_struct *s, int f_out)
 {
-	int i;
-	
-	/* tell the other guy how many we are going to be doing and how many
-	   bytes there are in the last chunk */
-	write_int(f_out,s?s->count:0);
-	write_int(f_out,s?s->n:block_size);
-	write_int(f_out,s?s->remainder:0);
+	if (s) {
+		size_t i;
 
-	if (!s) return;
+		/* tell the other guy how many we are going to be
+		   doing and how many bytes there are in the last
+		   chunk */
+		write_int(f_out, s->count);
+		write_int(f_out, s->n);
+		write_int(f_out, s->remainder);
 
-	for (i=0;i<s->count;i++) {
-		write_int(f_out,s->sums[i].sum1);
-		write_buf(f_out,s->sums[i].sum2,csum_length);
+		for (i = 0; i < s->count; i++) {
+			write_int(f_out, s->sums[i].sum1);
+			write_buf(f_out, s->sums[i].sum2, csum_length);
+		}
+	} else {
+		/* we don't have checksums */
+		write_int(f_out, 0);
+		write_int(f_out, block_size);
+		write_int(f_out, 0);
 	}
 }
 
-
 /*
   generate a stream of signatures/checksums that describe a buffer
 
@@ -375,6 +384,12 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 		return;
 	}
 
+	if (opt_ignore_existing && fnamecmp == fname) { 
+		if (verbose > 1)
+			rprintf(FINFO,"%s exists\n",fname);
+		return;
+	} 
+
 	if (update_only && cmp_modtime(st.st_mtime,file->modtime)>0 && fnamecmp == fname) {
 		if (verbose > 1)
 			rprintf(FINFO,"%s is newer\n",fname);
