@@ -1,7 +1,9 @@
 /* -*- c-file-style: "linux" -*-
    
+   rsync -- fast file replication program
+   
    Copyright (C) 1992-2001 by Andrew Tridgell <tridge@samba.org>
-   Copyright (C) 2001 by Martin Pool <mbp@samba.org>
+   Copyright (C) 2001, 2002 by Martin Pool <mbp@samba.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,10 +24,16 @@
  * @file socket.c
  * 
  * Socket functions used in rsync.
+ *
+ * This file is now converted to use the new-style getaddrinfo()
+ * interface, which supports IPv6 but is also supported on recent
+ * IPv4-only machines.  On systems that don't have that interface, we
+ * emulate it using the KAME implementation.
  **/
 
 #include "rsync.h"
 
+
 /* Establish a proxy connection on an open socket to a web roxy by
  * using the CONNECT method. */
 static int establish_proxy_connection(int fd, char *host, int port)
@@ -34,7 +42,7 @@ static int establish_proxy_connection(int fd, char *host, int port)
 	char *cp;
 
 	snprintf(buffer, sizeof(buffer), "CONNECT %s:%d HTTP/1.0\r\n\r\n", host, port);
-	if (write(fd, buffer, strlen(buffer)) != strlen(buffer)) {
+	if (write(fd, buffer, strlen(buffer)) != (int) strlen(buffer)) {
 		rprintf(FERROR, "failed to write to proxy: %s\n",
 			strerror(errno));
 		return -1;
@@ -105,7 +113,7 @@ int try_bind_local(int s,
 	bhints.ai_family = ai_family;
 	bhints.ai_socktype = ai_socktype;
 	bhints.ai_flags = AI_PASSIVE;
-	if (getaddrinfo(bind_address, NULL, &bhints, &bres_all) == -1) {
+	if ((error = getaddrinfo(bind_address, NULL, &bhints, &bres_all))) {
 		rprintf(FERROR, RSYNC_NAME ": getaddrinfo %s: %s\n",
 			bind_address, gai_strerror(error));
 		return -1;
@@ -150,7 +158,6 @@ int open_socket_out(char *host, int port, const char *bind_address,
 	int type = SOCK_STREAM;
 	int error;
 	int s;
-	int result;
 	struct addrinfo hints, *res0, *res;
 	char portbuf[10];
 	char *h;
@@ -382,7 +389,7 @@ void start_accept_loop(int port, int (*fn)(int ))
 		fd_set fds;
 		int fd;
 		struct sockaddr_storage addr;
-		int addrlen = sizeof(addr);
+		socklen_t addrlen = sizeof addr;
 
 		/* close log file before the potentially very long select so
 		   file can be trimmed by another process instead of growing
@@ -559,129 +566,6 @@ void become_daemon(void)
 	}
 }
 
-/**
- * Return the IP addr of the client as a string 
- **/
-char *client_addr(int fd)
-{
-	struct sockaddr_storage ss;
-	int     length = sizeof(ss);
-	static char addr_buf[100];
-	static int initialised;
-
-	if (initialised) return addr_buf;
-
-	initialised = 1;
-
-	if (getpeername(fd, (struct sockaddr *)&ss, &length)) {
-		exit_cleanup(RERR_SOCKETIO);
-	}
-
-	getnameinfo((struct sockaddr *)&ss, length,
-		addr_buf, sizeof(addr_buf), NULL, 0, NI_NUMERICHOST);
-	return addr_buf;
-}
-
-
-static int get_sockaddr_family(const struct sockaddr_storage *ss)
-{
-	return ((struct sockaddr *) ss)->sa_family;
-}
-
-
-/**
- * Return the DNS name of the client 
- **/
-char *client_name(int fd)
-{
-	struct sockaddr_storage ss;
-	int     length = sizeof(ss);
-	static char name_buf[100];
-	static char port_buf[100];
-	char *def = "UNKNOWN";
-	static int initialised;
-	struct addrinfo hints, *res, *res0;
-	int error;
-
-	if (initialised) return name_buf;
-
-	initialised = 1;
-
-	strcpy(name_buf,def);
-
-	if (getpeername(fd, (struct sockaddr *)&ss, &length)) {
-		/* FIXME: Can we really not continue? */
-		rprintf(FERROR, RSYNC_NAME ": getpeername on fd%d failed: %s\n",
-			fd, strerror(errno));
-		exit_cleanup(RERR_SOCKETIO);
-	}
-
-#ifdef INET6
-        if (get_sockaddr_family(&ss) == AF_INET6 && 
-	    IN6_IS_ADDR_V4MAPPED(&((struct sockaddr_in6 *)&ss)->sin6_addr)) {
-		struct sockaddr_in6 sin6;
-		struct sockaddr_in *sin;
-
-		memcpy(&sin6, &ss, sizeof(sin6));
-		sin = (struct sockaddr_in *)&ss;
-		memset(sin, 0, sizeof(*sin));
-		sin->sin_family = AF_INET;
-		length = sizeof(struct sockaddr_in);
-#ifdef HAVE_SOCKADDR_LEN
-		sin->sin_len = length;
-#endif
-		sin->sin_port = sin6.sin6_port;
-		memcpy(&sin->sin_addr, &sin6.sin6_addr.s6_addr[12],
-			sizeof(sin->sin_addr));
-        }
-#endif
-
-	/* reverse lookup */
-	if (getnameinfo((struct sockaddr *)&ss, length,
-			name_buf, sizeof(name_buf), port_buf, sizeof(port_buf),
-			NI_NAMEREQD | NI_NUMERICSERV) != 0) {
-		strcpy(name_buf, def);
-		rprintf(FERROR, "reverse name lookup failed\n");
-	}
-
-	/* forward lookup */
-	memset(&hints, 0, sizeof(hints));
-	hints.ai_family = PF_UNSPEC;
-	hints.ai_flags = AI_CANONNAME;
-	hints.ai_socktype = SOCK_STREAM;
-	error = getaddrinfo(name_buf, port_buf, &hints, &res0);
-	if (error) {
-		strcpy(name_buf, def);
-		rprintf(FERROR,
-			RSYNC_NAME ": forward name lookup for %s failed: %s\n",
-			port_buf,
-			gai_strerror(error));
-		return name_buf;
-	}
-
-	/* XXX sin6_flowinfo and other fields */
-	for (res = res0; res; res = res->ai_next) {
-		if (res->ai_family != get_sockaddr_family(&ss))
-			continue;
-		if (res->ai_addrlen != length)
-			continue;
-		if (memcmp(res->ai_addr, &ss, res->ai_addrlen) == 0)
-			break;
-	}
-
-	/* TODO: Do a  forward lookup as well to prevent spoofing */
-
-	if (res == NULL) {
-		strcpy(name_buf, def);
-		rprintf(FERROR, RSYNC_NAME ": "
-			"reverse name lookup mismatch on fd%d - spoofed address?\n",
-			fd);
-	}
-
-	freeaddrinfo(res0);
-	return name_buf;
-}
-
 
 /*******************************************************************
 this is like socketpair but uses tcp. It is used by the Samba
@@ -750,16 +634,20 @@ static int socketpair_tcp(int fd[2])
 }
 
 
-/*******************************************************************
-run a program on a local tcp socket, this is used to launch smbd
-when regression testing
-the return value is a socket which is attached to a subprocess
-running "prog". stdin and stdout are attached. stderr is left
-attached to the original stderr
- ******************************************************************/
+
+/**
+ * Run a program on a local tcp socket, so that we can talk to it's
+ * stdin and stdout.  This is used to fake a connection to a daemon
+ * for testing -- not for the normal case of running SSH.
+ *
+ * @return a socket which is attached to a subprocess running
+ * "prog". stdin and stdout are attached. stderr is left attached to
+ * the original stderr
+ **/
 int sock_exec(const char *prog)
 {
 	int fd[2];
+	
 	if (socketpair_tcp(fd) != 0) {
 		rprintf (FERROR, RSYNC_NAME
 			 ": socketpair_tcp failed (%s)\n",
@@ -772,10 +660,12 @@ int sock_exec(const char *prog)
 		close(1);
 		dup(fd[1]);
 		dup(fd[1]);
-		if (verbose > 3)
+		if (verbose > 3) {
+			/* Can't use rprintf because we've forked. */
 			fprintf (stderr,
 				 RSYNC_NAME ": execute socket program \"%s\"\n",
 				 prog);
+		}
 		exit (system (prog));
 	}
 	close (fd[1]);
